"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.ChannelWatcher = void 0;

var _simplexWatcher = require("./simplexWatcher");

var _snapTransaction = require("./snapTransaction");

function _classCallCheck(instance, Constructor) { if (!(instance instanceof Constructor)) { throw new TypeError("Cannot call a class as a function"); } }

function _defineProperties(target, props) { for (var i = 0; i < props.length; i++) { var descriptor = props[i]; descriptor.enumerable = descriptor.enumerable || false; descriptor.configurable = true; if ("value" in descriptor) descriptor.writable = true; Object.defineProperty(target, descriptor.key, descriptor); } }

function _createClass(Constructor, protoProps, staticProps) { if (protoProps) _defineProperties(Constructor.prototype, protoProps); if (staticProps) _defineProperties(Constructor, staticProps); return Constructor; }

var proposerMessageTypes = [_snapTransaction.SnapTransactionState.Proposing, _snapTransaction.SnapTransactionState.Accepted, _snapTransaction.SnapTransactionState.Rejected];

var ChannelWatcher = /*#__PURE__*/function () {
  function ChannelWatcher(ourStart, theirStart) {
    _classCallCheck(this, ChannelWatcher);

    this.us = new _simplexWatcher.SimplexWatcher(ourStart);
    this.them = new _simplexWatcher.SimplexWatcher(theirStart);
  }

  _createClass(ChannelWatcher, [{
    key: "setOurTrust",
    value: function setOurTrust(value) {
      this.ourTrust = value;
      this.updateTheirMax();
    }
  }, {
    key: "updateTheirMax",
    value: function updateTheirMax() {
      var ourTotal = this.us.getSum(false); // console.log("updateTheirMax", this.ourTrust, ourTotal);

      return this.them.setMax(this.ourTrust + ourTotal);
    }
  }, {
    key: "setTheirTrust",
    value: function setTheirTrust(value) {
      this.theirTrust = value;
      this.updateOurMax();
    }
  }, {
    key: "updateOurMax",
    value: function updateOurMax() {
      var theirTotal = this.us.getSum(false); // console.log("updateOurMax", this.theirTrust, theirTotal);

      return this.us.setMax(this.theirTrust + theirTotal);
    }
  }, {
    key: "getOurCurrent",
    value: function getOurCurrent() {
      var ourReceived = this.them.getSum(false);
      var ourUsed = this.us.getSum(true);
      return ourReceived - ourUsed;
    }
  }, {
    key: "getTheirCurrent",
    value: function getTheirCurrent() {
      var theirReceived = this.us.getSum(false);
      var theirUsed = this.them.getSum(true);
      return theirReceived - theirUsed;
    }
  }, {
    key: "getOurPayable",
    value: function getOurPayable() {
      return this.us.getSum(true, false);
    }
  }, {
    key: "getTheirPayable",
    value: function getTheirPayable() {
      return this.them.getSum(true, false);
    }
  }, {
    key: "getOurReceivable",
    value: function getOurReceivable() {
      return this.getTheirPayable();
    }
  }, {
    key: "getTheirReceivable",
    value: function getTheirReceivable() {
      return this.getOurPayable();
    }
  }, {
    key: "handleMessageWeSend",
    value: function handleMessageWeSend(stateTransition, time) {
      var proposerMessageTypes = [_snapTransaction.SnapTransactionState.Proposing, _snapTransaction.SnapTransactionState.Accepted, _snapTransaction.SnapTransactionState.Rejected];

      if (proposerMessageTypes.indexOf(stateTransition.newState) !== -1) {
        return this.us.handleProposerMessage(stateTransition, time);
      } else {
        var ret = this.them.handleDeciderMessage(stateTransition, time);

        if (stateTransition.newState === _snapTransaction.SnapTransactionState.Accepted) {
          this.updateOurMax();
        }

        return ret;
      }
    }
  }, {
    key: "handleMessageWeReceive",
    value: function handleMessageWeReceive(stateTransition, time) {
      if (proposerMessageTypes.indexOf(stateTransition.newState) !== -1) {
        return this.them.handleProposerMessage(stateTransition, time);
      } else {
        var ret = this.us.handleDeciderMessage(stateTransition, time);

        if (stateTransition.newState === _snapTransaction.SnapTransactionState.Accepted) {
          this.updateTheirMax();
        }

        return ret;
      }
    }
  }]);

  return ChannelWatcher;
}();

exports.ChannelWatcher = ChannelWatcher;
//# sourceMappingURL=channelWatcher.js.map