"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.SimplexWatcher = void 0;

var _hashlocks = require("hashlocks");

var _snapTransaction = require("./snapTransaction");

function _classCallCheck(instance, Constructor) { if (!(instance instanceof Constructor)) { throw new TypeError("Cannot call a class as a function"); } }

function _defineProperties(target, props) { for (var i = 0; i < props.length; i++) { var descriptor = props[i]; descriptor.enumerable = descriptor.enumerable || false; descriptor.configurable = true; if ("value" in descriptor) descriptor.writable = true; Object.defineProperty(target, descriptor.key, descriptor); } }

function _createClass(Constructor, protoProps, staticProps) { if (protoProps) _defineProperties(Constructor.prototype, protoProps); if (staticProps) _defineProperties(Constructor, staticProps); return Constructor; }

function expired(expiresAt, sentAt) {
  return expiresAt > sentAt;
}

var SimplexWatcher = /*#__PURE__*/function () {
  function SimplexWatcher() {
    var start = arguments.length > 0 && arguments[0] !== undefined ? arguments[0] : 0;
    var max = arguments.length > 1 && arguments[1] !== undefined ? arguments[1] : 0;

    _classCallCheck(this, SimplexWatcher);

    this.start = start;
    this.max = max;
    this.entries = [];
  }

  _createClass(SimplexWatcher, [{
    key: "handleProposerMessage",
    value: function handleProposerMessage(msg, time) {
      switch (msg.newState) {
        case _snapTransaction.SnapTransactionState.Proposing:
          if (this.entries[msg.transId] === undefined) {
            // CHECK 1: only add a new proposal if it doesn't bring the total over max.
            if (msg.amount + this.getSum(true) > this.max) {
              console.log(msg, this);
              throw new Error("Amount would bring total over max");
            }

            var trans = {
              amount: msg.amount
            };

            if (msg.condition) {
              trans.condition = msg.condition;
            }

            if (msg.expiresAt) {
              trans.expiresAt = msg.expiresAt;
            }

            this.entries[msg.transId] = {
              status: msg.newState,
              trans: trans,
              time: time
            };
          }

          break;

        case _snapTransaction.SnapTransactionState.Accepted:
          if (this.entries[msg.transId] !== undefined && this.entries[msg.transId].status === _snapTransaction.SnapTransactionState.Accepting) {
            this.entries[msg.transId].status = _snapTransaction.SnapTransactionState.Accepted;
          }

          break;

        case _snapTransaction.SnapTransactionState.Rejected:
          if (this.entries[msg.transId] !== undefined && this.entries[msg.transId].status === _snapTransaction.SnapTransactionState.Rejecting) {
            this.entries[msg.transId].status = _snapTransaction.SnapTransactionState.Rejected;
          }

          break;
      }
    }
  }, {
    key: "handleDeciderMessage",
    value: function handleDeciderMessage(msg, time) {
      switch (msg.newState) {
        case _snapTransaction.SnapTransactionState.Proposed:
          if (this.entries[msg.transId] !== undefined && this.entries[msg.transId].status === _snapTransaction.SnapTransactionState.Proposing) {
            this.entries[msg.transId].status = _snapTransaction.SnapTransactionState.Proposed;
          }

          break;

        case _snapTransaction.SnapTransactionState.Accepting:
          if (this.entries[msg.transId] !== undefined && (this.entries[msg.transId].status === _snapTransaction.SnapTransactionState.Proposing || this.entries[msg.transId].status === _snapTransaction.SnapTransactionState.Proposed)) {
            // CHECK 2: only commit an accept of a conditional transaction if the preimage is given correctly
            if (this.entries[msg.transId].trans.condition && (0, _hashlocks.sha256)(msg.preimage) !== this.entries[msg.transId].trans.condition) {
              return;
            } // CHECK 3: only commit a transaction with expiresAt if that time hasn't passed yet


            if (this.entries[msg.transId].trans.expiresAt && expired(this.entries[msg.transId].trans.expiresAt, time)) {
              return;
            }

            this.entries[msg.transId].status = _snapTransaction.SnapTransactionState.Accepting;
          }

          break;

        case _snapTransaction.SnapTransactionState.Rejecting:
          if (this.entries[msg.transId] !== undefined && (this.entries[msg.transId].status === _snapTransaction.SnapTransactionState.Proposing || this.entries[msg.transId].status === _snapTransaction.SnapTransactionState.Proposed)) {
            this.entries[msg.transId].status = _snapTransaction.SnapTransactionState.Accepting;
          }

          break;
      }
    }
  }, {
    key: "getSum",
    value: function getSum(includePending) {
      var includeAccepted = arguments.length > 1 && arguments[1] !== undefined ? arguments[1] : true;
      var includeRejected = arguments.length > 2 && arguments[2] !== undefined ? arguments[2] : false;
      var statusAccepted = _snapTransaction.SnapTransactionState.Accepted;
      var statusesPending = [_snapTransaction.SnapTransactionState.Proposing, _snapTransaction.SnapTransactionState.Proposed, _snapTransaction.SnapTransactionState.Rejecting, _snapTransaction.SnapTransactionState.Accepting];
      var statusRejected = _snapTransaction.SnapTransactionState.Rejected;
      var total = this.start;
      var entriesToInclude = this.entries.filter(function (currentEntry) {
        if (includeAccepted && currentEntry.status === statusAccepted) {
          return true;
        }

        if (includePending && statusesPending.indexOf(currentEntry.status) !== -1) {
          return true;
        }

        if (includeRejected && currentEntry.status === statusRejected) {
          return true;
        }
      });
      entriesToInclude.forEach(function (currentEntry) {
        total += currentEntry.trans.amount;
      });
      return total;
    }
  }, {
    key: "setMax",
    value: function setMax(value) {
      // console.log("setting max", value);
      this.max = value;
    }
  }]);

  return SimplexWatcher;
}();

exports.SimplexWatcher = SimplexWatcher;
//# sourceMappingURL=simplexWatcher.js.map