"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.SnapServer = void 0;

var _channelWatcher = require("./channelWatcher");

function _classCallCheck(instance, Constructor) { if (!(instance instanceof Constructor)) { throw new TypeError("Cannot call a class as a function"); } }

function _defineProperties(target, props) { for (var i = 0; i < props.length; i++) { var descriptor = props[i]; descriptor.enumerable = descriptor.enumerable || false; descriptor.configurable = true; if ("value" in descriptor) descriptor.writable = true; Object.defineProperty(target, descriptor.key, descriptor); } }

function _createClass(Constructor, protoProps, staticProps) { if (protoProps) _defineProperties(Constructor.prototype, protoProps); if (staticProps) _defineProperties(Constructor, staticProps); return Constructor; }

var SnapServer = /*#__PURE__*/function () {
  function SnapServer(locals) {
    var _this = this;

    _classCallCheck(this, SnapServer);

    this.msgLog = [];
    this.channelWatchers = {};
    locals.forEach(function (agentName) {
      _this.channelWatchers[agentName] = {};
    });
  }

  _createClass(SnapServer, [{
    key: "getChannelWatcher",
    value: function getChannelWatcher(agentName, peerName, unit) {
      var agentStart = arguments.length > 3 && arguments[3] !== undefined ? arguments[3] : 0;
      var peerStart = arguments.length > 4 && arguments[4] !== undefined ? arguments[4] : 0;

      if (!this.channelWatchers[agentName]) {
        // this.channelWatchers[agentName] = {};
        throw new Error("Agent not local! " + agentName);
      }

      if (!this.channelWatchers[agentName][peerName]) {
        this.channelWatchers[agentName][peerName] = {};
      }

      if (!this.channelWatchers[agentName][peerName][unit]) {
        // console.log(
        //   "creating channel watcher",
        //   agentName,
        //   peerName,
        //   unit,
        //   agentStart,
        //   peerStart
        // );
        this.channelWatchers[agentName][peerName][unit] = new _channelWatcher.ChannelWatcher(agentStart, peerStart);
      } // console.log(
      //   "returning channel watcher",
      //   agentName,
      //   peerName,
      //   unit,
      //   agentStart,
      //   peerStart,
      //   this.channelWatchers[agentName][peerName][unit]
      // );


      return this.channelWatchers[agentName][peerName][unit];
    }
  }, {
    key: "isLocal",
    value: function isLocal(agentName) {
      return typeof this.channelWatchers[agentName] !== "undefined";
    }
  }, {
    key: "getBalances",
    value: function getBalances(agentName, peerName, unit) {
      var channelWatcher = this.getChannelWatcher(agentName, peerName, unit);
      return {
        current: channelWatcher.getOurCurrent(),
        payable: channelWatcher.getOurPayable(),
        receivable: channelWatcher.getOurReceivable()
      };
    }
  }, {
    key: "setStartBalance",
    value: function setStartBalance(agentName, peerName, unit, agentStart, peerStart) {
      this.getChannelWatcher(agentName, peerName, unit, agentStart, peerStart);
    }
  }, {
    key: "processTrustChange",
    value: function processTrustChange(msg) {
      if (this.isLocal(msg.from)) {
        // console.log(
        //   "set our trust",
        //   msg.from,
        //   msg.to,
        //   msg.unit,
        //   msg.newTrustLevel
        // );
        this.getChannelWatcher(msg.from, msg.to, msg.unit).setOurTrust(msg.newTrustLevel);
      }

      if (this.isLocal(msg.to)) {
        // console.log(
        //   "set their trust",
        //   msg.to,
        //   msg.from,
        //   msg.unit,
        //   msg.newTrustLevel
        // );
        this.getChannelWatcher(msg.to, msg.from, msg.unit).setTheirTrust(msg.newTrustLevel);
      }
    }
  }, {
    key: "processSnapMessage",
    value: function processSnapMessage(msg) {
      if (this.isLocal(msg.from)) {
        this.getChannelWatcher(msg.from, msg.to, msg.unit).handleMessageWeSend(msg.stateTransition, msg.time);
      }

      if (this.isLocal(msg.to)) {
        this.getChannelWatcher(msg.to, msg.from, msg.unit).handleMessageWeReceive(msg.stateTransition, msg.time);
      }
    }
  }, {
    key: "logMessage",
    value: function logMessage(msg) {
      if (this.msgLog.length && msg.time < this.msgLog[this.msgLog.length - 1].time) {
        console.log("timing?", msg, this.msgLog);
        throw new Error("Please log messages in chronological order");
      }

      this.msgLog.push(msg);

      if (msg.stateTransition) {
        return this.processSnapMessage(msg);
      }

      if (msg.newTrustLevel) {
        return this.processTrustChange(msg);
      }
    }
  }]);

  return SnapServer;
}();

exports.SnapServer = SnapServer;
//# sourceMappingURL=snapServer.js.map